#-------------------------------------------------------------------------------
import pygame
import debug
import data
import config
import player
import level
import music
import hook
import objects
import bitmaps
import gui
from pygame.locals import *

#-------------------------------------------------------------------------------
class GameData:
    level = 0
    lifes = config.MAX_LIFES

#-------------------------------------------------------------------------------
class World:
    END_GAME = 0
    PLAYER_DEAD = 1
    END_LEVEL = 2
    #---------------------------------------------------------------------------
    def __init__( self ):
        self.screen = pygame.display.get_surface()
        self.background = self.screen.copy()
        self.background.fill( [50,50,50] )

        self.sprites = (
            pygame.sprite.RenderUpdates(),
            pygame.sprite.RenderUpdates(),
            pygame.sprite.RenderUpdates() )

        self.gui = pygame.sprite.RenderUpdates()

        self.fps = debug.Variable( 
            config.WIDTH - 70, debug.nexty(), 'fps' )
        self.numObjects = debug.Variable( 
            config.WIDTH - 80, debug.nexty(), '#objs' )

        self.level = level.Level( self, GameData.level )
        self.level.draw( self.background )

        playerPos = self.level.getPlayerBegin()
        playerPos = (playerPos[0]*32 + 96, playerPos[1]*32)
        self.myPlayer = player.Player( playerPos )
        self.myPlayer.setLevel( self.level )
        self.myPlayer.setWorld( self )
        self.myPlayer.setStringAmount( self.level.getDefaultStringAmount() )
        
        self.sprites[1].add( self.myPlayer )

        self.stringCounter = gui.StringCounter()
        self.stringCounter.writeNumber( self.myPlayer.getStringAmount() )
        self.gui.add( self.stringCounter )

        self.lifeCounter = gui.LifeCounter( config.MAX_LIFES )
        self.lifeCounter.setLifes( GameData.lifes )
        self.gui.add( self.lifeCounter )

        self.worldExit = World.END_GAME

    #---------------------------------------------------------------------------
    def run( self ):
        self.running = True
        while self.running:
            init_tick = pygame.time.get_ticks()

            self.update( init_tick )
            self.draw()
            
            self.numObjects.setValue( sum( map( len, self.sprites ) ) )
                        
            end_tick = pygame.time.get_ticks()
            diff_tick = end_tick - init_tick
            if diff_tick > 0:
                self.fps.setValue( 1000 / diff_tick )
            if diff_tick < config.TIME_STEP:
                pygame.time.delay( diff_tick )
            else:
                debug.printMsg( 
                    "frame too slow " + str( diff_tick ) 
                    + " " + str( config.TIME_STEP ) )

        return self.worldExit

    #---------------------------------------------------------------------------
    def endWorld( self, code ):
        GameData.lifes = self.lifeCounter.lifes
        self.running = False
        self.worldExit = code

    #---------------------------------------------------------------------------
    def update( self, current ):
        shotHook = False
        pygame.event.pump()
        events = pygame.event.get()
        for event in events:
            if event.type == QUIT:
                self.endWorld( World.END_GAME )
            elif event.type == MOUSEMOTION:
                pass
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    self.endWorld( World.END_GAME )
                elif event.key == K_SPACE:
                    shotHook = True
            elif event.type == KEYUP:
                pass
            else:
                debug.printMsg( str( event ) )

        if shotHook:
            self.myPlayer.stop()
            self.myPlayer.hookShot()
        else:
            keys = pygame.key.get_pressed()
            movement = ''
            if keys[K_UP]:
                movement = 'up'
            if keys[K_LEFT]:
                movement = 'left'
            if keys[K_DOWN]:
                movement = 'down'
            if keys[K_RIGHT]:
                movement = 'right'        

            if movement == '':
                self.myPlayer.stop()
            else:
                self.myPlayer.moveTo( movement )

        map( lambda x: x.update( current ), self.sprites )
        
        self.stringCounter.writeNumber( self.myPlayer.getStringAmount() )
        self.gui.update( current )

        for spriteList in self.sprites:
            for spr in spriteList:
                if spr.isZombie():
                    debug.printMsg( str( spr ) + "removed Zombie" )
                    spriteList.remove( spr )

        if self.myPlayer.isFullDead():
            debug.printMsg( "Player died" )
            playerPos = self.level.getPlayerBegin()
            playerPos = (playerPos[0]*32 + 96, playerPos[1]*32)
            self.lifeCounter.setLifes( self.lifeCounter.lifes - 1 )
            if self.lifeCounter.lifes <= 0:
                self.endWorld( World.PLAYER_DEAD )
            self.myPlayer.spawn( playerPos )
            self.myPlayer.setStringAmount( 
                self.level.getDefaultStringAmount() )

    #---------------------------------------------------------------------------
    def draw( self ):
        self.screen.blit( self.background, [0,0] )

        map( lambda x: x.draw( self.screen ), self.sprites )
        self.gui.draw( self.screen )

        debug.update()
        pygame.display.update()
        
    #---------------------------------------------------------------------------
    def insertSprite( self, newSpr ):
        self.sprites[1].add( newSpr )

    #---------------------------------------------------------------------------
    def insertObjSprite( self, newSpr, bottom=True ):
        if bottom:
            self.sprites[0].add( newSpr )
        else:
            self.sprites[2].add( newSpr )

    #---------------------------------------------------------------------------
    def locateHookLog( self, x, y ):
        for spriteList in self.sprites:
            for spr in spriteList:
                if spr.typename == 'HookLog':
                    point = spr.getHookPoint()
                    if point.collidepoint( x, y ):
                        return spr
        return None

    #---------------------------------------------------------------------------
    def locateHookPoint( self, x, y ):
        for spriteList in self.sprites:
            for spr in spriteList:
                if spr.isHookable():
                    point = spr.getHookPoint()
                    if point.collidepoint( x, y ):
                        return spr
        return None

    #---------------------------------------------------------------------------
    def createStringBridge( self, log1, log2 ):
        bridge = objects.StringBridge( log1, log2, self.level )
        self.sprites[0].add( bridge )

    #---------------------------------------------------------------------------
    def checkCollision( self, player, rect ):
        result = False
        for spriteList in self.sprites:
            for spr in spriteList:
                result = result or  spr.checkCollision( player, rect )
        return result

    #---------------------------------------------------------------------------
    def exitLevel( self ):
        self.endWorld( World.END_LEVEL )

#-------------------------------------------------------------------------------
def resetGameData():
    GameData.level = 0
    GameData.lifes = config.MAX_LIFES
    
#-------------------------------------------------------------------------------
